Ext.namespace('dSS11', 'dSS11.sys');

dSS11.sys.FormatSDForm = Ext.extend(Ext.form.FormPanel,
{
    initComponent: function()
    {
        Ext.apply(this, {
            layout: 'fit',
            bodyBorder: false,
            items:
            [
                {
                    xtype: 'fieldset',
                    ref: 'mainSet',
                    title: _("SD Card"),
                    layout: 'form',
                    height: 168,
                    scope: this,
                    items:
                    [
                        {
                            xtype: 'label',
                            cls: 'x-green-text',
                            html: _("The dSS includes a micro-SD card to " +
                                    "store metering and logging data.") +
                                  _("Formatting the card will delete metering" +
                                    " and logging data, but not the system " +
                                    "settings.") + '<br/>' +
                                  _("It is recommented to format the card " +
                                    "if errors were detected.")
                        },
                        {
                            xtype: 'box',
                            height: 10
                        },
                        {
                            xtype: 'label',
                            cls: 'x-green-text',
                            ref: 'cardStatus',
                            html: _("SD card status") + ':'
                        },
                        {
                            xtype: 'box',
                            height: 10
                        },
                        {
                            xtype: 'button',
                            cls: 'x-alignright',
                            text: _("Format SD card"),
                            minWidth: 110,
                            handler: this.formatSD.createDelegate(this)
                        }
                    ]
                }
            ]
        });

        dSS11.sys.FormatSDForm.superclass.initComponent.apply(this, arguments);

    },
    formatWaitMsg: function(secs, w) {
        if (secs > 0) {
            w.updateProgress(1-(secs/(dSS11.config.rebootWait*3)), secs + " " +
                    dSS11.config.gt.ngettext("second remaining",
                                             "seconds remaining",
                                              secs));
            secs--;
            this.formatWaitMsg.defer(1000, this, [secs, w]);
        } else {
            w.hide();
            this.findParentByType(Ext.Viewport).el.mask(_("Reloading page") + "...");
            window.location = window.location.protocol + '//' +
                              window.location.host;

        }
    },
    formatSD: function() {
        var maskEl = this.findParentByType('viewport').getEl();
        Ext.Msg.show({
            title: _("Format SD card"),
            msg: _("Are you sure that you want to format the SD card?") +
                   "<br/>" +
                   _("Note: the system will be rebooted."),
                buttons: Ext.Msg.YESNO,
                scope: this,
                fn: function(button, text) {
                    if (button == 'yes') {
                        Ext.Ajax.request({
                            url: '/cgi-bin/formatsdwrapper.sh',
                            disableCaching: true,
                            params: { format : 'sdcard' },
                            method: 'GET',
                            scope: this,
                            success: function(response) {
                                try {
                                    var jsonData = Ext.util.JSON.decode(
                                            response.responseText);

                                    if (jsonData.success !== true) {
                                        Ext.Msg.alert(_("Error"),
                                                        jsonData.message);
                                        maskEl.unmask();
                                        return;
                                    } else {
                                        var w = Ext.Msg.progress(
                                            _("Formatting SD card"),
                                            _("Please wait") +
                                              "<br/>" +
                                            _("The setup page will be " +
                                              "automatically reloaded " +
                                              "after the time elapses"));
                                        this.formatWaitMsg(
                                                dSS11.config.rebootWait*3, w);
                                    }
                                } catch (error) {
                                    Ext.Msg.alert(_("Error"),
                                        _("SD card format could not " +
                                          "be initiated") + ": " +
                                            error.message);
                                }
                            },
                            failure: function() {
                                Ext.Msg.alert(_("Error"),
                                    _("SD card format could not be " +
                                      "initiated"));
                                maskEl.unmask();
                            }
                        });
                    }
                },
                animEl: 'elId',
                icon: Ext.MessageBox.QUESTION
        });
    },
    reloadData: function() {
        var maskEl = this.mainSet.ownerCt.el;
        maskEl.mask(_("Loading SD card information") + "...", 'x-mask-loading');
        Ext.Ajax.request({
            url: '/cgi-bin/sdcardinfo.sh',
            method: "GET",
            scope: this,
            disableCaching: true,
            success: function(response) {
                maskEl.unmask();
                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);

                    if (jsonData.success !== true) {
                        Ext.Msg.alert(_("Error"), jsonData.message);
                        return;
                    }

                    this.mainSet.cardStatus.update('<b>' +
                                                    _("SD card status") +
                                                    '</b>' +
                                                    ': ' + _(jsonData.info));
                    
                } catch (error) {
                    Ext.Msg.alert(_("Error"),
                                  _("Could not load SD card information") +
                                    ": "  + error.message);
                }
            },
            failure: function(response) {
                var errMsg = _("Could not load SD card information");
                try {
                    var json = Ext.util.JSON.decode(response.responseText);
                    if (json.message) {
                        errMsg = ": " + _(json.message);
                    }
                } catch (err) {}
                Ext.Msg.alert(_("Error"),
                              _("Could not load SD card information") +
                                errMsg);
                this.mainSet.cardStatus.update('<b>' +
                                                _("SD card status") +
                                                '</b>' + ': ' + "unknown");
                this.mainSet.ownerCt.el.unmask();
            }
        });
    }
});

Ext.reg('dss11formatsd', dSS11.sys.FormatSDForm);

