Ext.namespace('dSS11', 'dSS11.config');

dSS11.SystemUpdatePanel = Ext.extend(Ext.Panel,
    {
        initComponent: function()
        {
            this.updateStore = new Ext.data.JsonStore(
                {
                    url: '/cgi-bin/opkgwrapper.sh?action=list-upgradable',
                    root: 'data',
                    fields: 
                    [
                        'name', 'version', 'maintainer', 
                        'description', 'homepage', 'state'
                    ]
                }
            );

            this.loaded = false;
            this.fromRefetch = false;

            // reload UI if at least one of those packages is part the upgrade
            this.reloadList = [];
            this.reloadList = this.reloadList.concat(
                                            dSS11.config.reloadUIAfterUpdate);

            var tpl = dSS11.util.defaultIPKTemplate();
            tpl.compile();

            this.mainGrid = new Ext.grid.GridPanel(
                {
                    store: this.updateStore,
                    enableColumnHide: false,
                    enableHdMenu: false,
                    columns:
                    [
                        {
                            xtype: 'templatecolumn',
                            header: _("Available updates"),
                            dataIndex: 'name',
                            sortable: true,

                            tpl: tpl
                        }
                    ],
                    trackMouseOver: false,
                    viewConfig: { forceFit: true },
                    disableSelection: true,
                    bodyBorder: false
                }
            );

            this.noUpdatesTooltip =  new Ext.ToolTip(
                {
                    target: 'dSS11SysUpdateButton',
                    html: '<br/>' + 
                          _("Your system is up to date"),
                    title: _("No updates found"),
                    closable: true,
                    draggable: true,
                    anchor: 'right'
                }
            );

            this.noDSMUpdatesTooltip =  new Ext.ToolTip(
                {
                    target: 'dSS11dSMUpdateButton',
                    html: '<br/>' +
                          _("All dSMs are up to date"),
                    title: _("No updates found"),
                    closable: true,
                    draggable: true,
                    anchor: 'left'
                }
            );

            this.gotUpdatesTooltip = new Ext.ToolTip(
                {
                    target: 'dSS11SysUpdateButton',
                    html: '<br/>' +
                          _("Updates are available for your system, click the update button to install them"),
                    title: _("Updates found"),
                    closable: true,
                    draggable: true,
                    anchor: 'right'
                }
            );

            this.gotDSMUpdatesTooltip = new Ext.ToolTip(
                {
                    target: 'dSS11dSMUpdateButton',
                    html: '<br/>' + 
                          _("New dSM firmware is available, click the update button to start the firmware upgrade"),
                    title: _("Updates found"),
                    closable: true,
                    draggable: true,
                    anchor: 'left'
                }
            );


            Ext.apply(this,
                {
                    layout: 'fit',
                    tbar: [
                        {
                            xtype: 'button',
                            text: _("Check for new updates"),
                            iconCls: 'icon-refresh-feed',
                            scope: this,
                            tooltip: _("Contact the server and check availability of new updates"),
                            handler: this.handleRefetch.createDelegate(this)
                        },
                        {
                            xtype: 'button',
                            text: _("Update system"),
                            ref: 'dSS11SysUpdateButton',
                            id: 'dSS11SysUpdateButton',
                            iconCls: 'icon-package-update',
                            disabled: true,
                            scope: this,
                            handler: this.updateSystem.createDelegate(this)
                        },
                        {
                            xtype: 'button',
                            text: _("Update dSM firmware"),
                            ref: 'dSS11dSMUpdateButton',
                            id: 'dSS11dSMUpdateButton',
                            iconCls: 'icon-package-update',
                            disabled: true,
                            scope: this,
                            tooltip: _("Update dSM firmware"),
                            handler: this.updateDSMFirmware.createDelegate(this)
                        },
                        '->',
                        {
                            xtype: 'button',
                            text: _("Release notes"),
                            iconCls: 'icon-release-notes',
                            scope: this,
                            tooltip: _("Display release information"),
                            handler: this.showReleaseNotes.createDelegate(this)
                        }
                    ],
                    items: [ this.mainGrid ]
                }
            );

            dSS11.SystemUpdatePanel.superclass.initComponent.apply(this, arguments);

            this.on('activate',
                function() 
                {
                    if (this.loaded === false) {
                        if (dSS11.config.updateInProgress !== true) {
                            this.updateStore.load();
                            this.loaded = true;
                            var p = Ext.urlDecode(
                                        window.location.search.substring(1));
                            if (p.showupdate == 1) {
                                this.handleRefetch();
                            }

                        } else {
                           var fn =
                                dSS11.util.checkUpdateProgress.createDelegate(
                                    this,
                                    [
                                        'system',
                                        this.findParentByType('viewport').el,
                                        this.updateDone.createDelegate(this)
                                    ]);
                            fn();
                            this.findParentByType('viewport').el.mask(
                                _("Performing update, this may take several " +
                                  "minutes") + "...", 'x-mask-loading');
                        }
                    }
                },
                this);

            this.on('hide',
                function()
                {
                    this.noUpdatesTooltip.hide();
                    this.gotUpdatesTooltip.hide();
                    this.noDSMUpdatesTooltip.hide();
                    this.gotDSMUpdatesTooltip.hide();
                },
                this);

            dSS11.MessageBus.on('refreshsystemupdateview', function() {
                    if (this.isVisible()) {
                        this.refreshList();
                    }
                },
                this);

            this.updateStore.on('load',
                function(store, records)
                {
                    var button = this.getTopToolbar().get('dSS11SysUpdateButton');
                    var count = this.updateStore.getTotalCount();

                    if (count > 0) {
                        this.noUpdatesTooltip.hide();
                        this.noUpdatesTooltip.disable();
                        this.gotUpdatesTooltip.enable();
                        button.enable();
                        this.gotUpdatesTooltip.setTitle(
                                count + " " + 
                                dSS11.config.gt.npgettext('noun', "Update",
                                                    "n% Updates", count) +
                                        " " + _("found"));

                        if (this.isVisible()) {
                            this.gotUpdatesTooltip.show();
                        }

                        this.reloadAfterUpdate = false;
                        var j;
                        for (j = 0; j < records.length; j++) {
                            var pkg = records[j].get('name');
                            var i;
                            for (i = 0; i < this.reloadList.length; i++) {
                                if (pkg === this.reloadList[i]) {
                                    this.reloadAfterUpdate = true;
                                    break;
                                }
                            }
                        }
                    } else {
                        button.disable();
                        this.gotUpdatesTooltip.hide();
                        this.gotUpdatesTooltip.disable();
                        this.noUpdatesTooltip.enable();
                        if ((this.fromRefetch === true) && this.isVisible()) {
                            this.noUpdatesTooltip.show();
                            this.fromRefetch = false;
                        }
                    }
                    this.findParentByType('viewport').el.unmask();
                },
                this);

        },
        refreshList: function() 
        {
            this.updateStore.load();
        },
        updateDone: function(jsonData)
        {
            var msg = _(jsonData.message);
            var title = _("Info");
            if (jsonData.code === dSS11.config.codes.ERR_OK) {
                msg = _("Your system was successfully updated.");
            } else if (jsonData.code === dSS11.config.codes.ERR_OPKG) {
                if (!(jsonData.opkgcode &
                    ~(dSS11.config.opkgcodes.ERR_PACKAGE_ALREADY_INSTALLED |
                      dSS11.config.opkgcodes.ERR_PACKAGE_NOT_DOWNGRADED))) {
                    // these are errors that we can usually ignore
                    msg = _("Your system was successfully updated.");
                } else if (jsonData.opkgcode ===
                           dSS11.config.opkgcodes.ERR_DOWNLOAD_FAILED) {
                    title = _("Error");
                    msg = _("Failed to download packages, please try again " +
                            "later!");
                } else {
                    title = _("Error");
                }
            }

            if (dSS11.config.updateInProgress === true) {
                this.reloadAfterUpdate = true;
            }

            if (this.reloadAfterUpdate === true) {
                msg = msg + ' ' + _("The web UI needs to be reloaded.");
            }

            dSS11.StatusBar.setMessage(msg);
            Ext.Msg.show({
                title: title,
                msg: msg,
                icon: Ext.MessageBox.INFO,
                buttons: Ext.Msg.OK,
                scope: this,
                fn: function() {
                    if (this.reloadAfterUpdate === true) {
                        this.findParentByType(Ext.Viewport).el.mask(
                                            _("Reloading page") + "...");
                        var p = Ext.urlDecode(
                                        window.location.search.substring(1));
                        if (p.showupdate == 1) {
                            window.location.reload(true);
                        } else {
                            window.location = window.location + '?showupdate=1'
                        }
                    }
                }
            });
            this.refreshList();
            var fn = dSS11.util.dSMFW.createDelegate(this,
                    [ 'check',
                      undefined,
                      this.dSMHandleFWUpdateButton.createDelegate(this) ]);
            fn();
        },
        dSMUpgradeFirmwareCallback: function()
        {
            var button = this.getTopToolbar().get('dSS11dSMUpdateButton');
            button.disable();
            var msg = _("dSM firmware has been upgraded");
            dSS11.StatusBar.setMessage(msg);
            Ext.Msg.show({
                title: _("Info"),
                msg: msg,
                icon: Ext.MessageBox.INFO,
                buttons: Ext.Msg.OK
            });
        },
        updateDSMFirmware: function()
        {
            var fn = dSS11.util.dSMFW.createDelegate(this,
                    [ 'upgrade',
                      this.findParentByType('viewport').el,
                      this.dSMUpgradeFirmwareCallback.createDelegate(this) ]);
            fn();
        },
        dSMHandleFWUpdateButton: function(dSMs)
        {
            var button = this.getTopToolbar().get('dSS11dSMUpdateButton');
            if (dSMs > 0) {
                this.noDSMUpdatesTooltip.disable();
                this.noDSMUpdatesTooltip.hide();
                this.gotDSMUpdatesTooltip.enable();
                button.enable();
                this.gotDSMUpdatesTooltip.setTitle(
                        dSS11.config.gt.strargs(
                        dSS11.config.gt.ngettext("%1 dSM needs upgrading",
                                            "%1 dSMs need upgrading", dSMs),
                        [ dSMs ]));
                if (this.isVisible()) {
                    this.gotDSMUpdatesTooltip.show();
                }
            } else {
                button.disable();
                this.gotDSMUpdatesTooltip.disable();
                this.gotDSMUpdatesTooltip.hide();
                this.noDSMUpdatesTooltip.enable();
                if (this.isVisible() && (dSMs === 0)) {
                    this.noDSMUpdatesTooltip.show();
                }
            }
        },
        dSMCheckFirmwareAndFeed: function(dSMs)
        {
            var fn = dSS11.util.opkg.createDelegate(this,
                           [ 'update',
                              this.findParentByType('viewport').el,
                              this.refetchDone.createDelegate(this) ]);
            fn();
            this.dSMHandleFWUpdateButton(dSMs);
        },
        refetchDone: function()
        {
            this.findParentByType('viewport').el.mask();
            this.fromRefetch = true;
            this.refreshList();
        },
        handleRefetch: function()
        {
            var fn = dSS11.util.dSMFW.createDelegate(this,
                    [ 'check',
                      this.findParentByType('viewport').el,
                      this.dSMCheckFirmwareAndFeed.createDelegate(this) ]);
            fn();
        },
        updateSystem: function() {
            var fn = dSS11.util.opkg.createDelegate(this,
                           [ 'upgrade',
                              this.findParentByType('viewport').el,
                              this.updateDone.createDelegate(this) ]);
            fn();
        },
        showReleaseNotes: function() {
            var maskEl = this.findParentByType('viewport').el;
            maskEl.mask();
            var win = new Ext.Window(
            {
                title: _("Release notes"),
                width: 640,
                height: 480,
                layout: 'fit',
                listeners:
                {
                    close: function()
                    {
                        maskEl.unmask();
                    }
                },
                items:
                [
                    {
                        xtype: 'panel',
                        border: false,
                        autoScroll: true,
                        autoLoad: '/releasenotes/notes.html'
                    }
                ],
                buttons:
                [
                    {
                        text: _("Close"),
                        scope: this,
                        handler: function() {
                            win.close();
                        }
                    }
                ]
            });
            win.show();
        }
    }
);

Ext.reg('dss11systemupdate', dSS11.SystemUpdatePanel);

